import argparse

from jumbo_api import JumboApi


def main():
    """Main function."""
    parser = argparse.ArgumentParser(description="Run some live tests against the API")
    parser.add_argument(
        'username', type=str,
        help="Your email address")
    parser.add_argument(
        'password', type=str,
        help="Your password")
    args = parser.parse_args()
    username = args.username
    password = args.password
    api = JumboApi(username, password)

    print("PROFILE")
    profile = api.get_profile()
    print(profile)

    print("")
    print("OPEN DELIVERIES")
    deliveries = api.get_open_deliveries()
    for delivery in deliveries:
        print(delivery)

    print("")
    print("OPEN PICK UPS")
    pick_ups = api.get_open_pick_ups()
    for pick_up in pick_ups:
        print(pick_up)

    print("")
    print("BASKET")
    basket = api.get_basket()
    print(basket)

    print("")
    print("OPEN DELIVERY TIME SLOTS")
    time_slots = api.get_open_delivery_time_slots()
    for time_slot in time_slots:
        print(time_slot)

    print("")
    print("OPEN PICK UP TIME SLOTS")
    time_slots = api.get_open_pick_up_time_slots()
    for time_slot in time_slots:
        print(time_slot)


if __name__ == '__main__':
    main()



from objects.store import Store


class Profile(object):
    def __init__(self, data):
        self.id = data.get("identifier")
        self.store = Store(data.get("store"))

    def __str__(self):
        return f"{self.id} {self.store}"



class Price(object):
    def __init__(self, data):
        self.currency = data.get("currency")
        self.amount = data.get("amount")
        self.format = self.currency + " " + '{:.2f}'.format(self.amount / 100)

    def __str__(self):
        return f"{self.currency} {self.amount} {self.format}"



from objects.price import Price


class Basket(object):
    def __init__(self, data):
        self.amount = len(data.get('items'))
        self.price = Price(data.get("prices").get('total'))

    def __str__(self):
        return f"{self.amount} {self.price}"



""" Python wrapper for the Jumbo API """

import logging
import time

import requests

from objects.basket import Basket
from objects.delivery import Delivery
from objects.pick_up import PickUp
from objects.profile import Profile
from objects.time_slot import TimeSlot

BASE_URL = 'https://mobileapi.jumbo.com/'
VERSION = 'v9'

AUTHENTICATE_URL = BASE_URL + VERSION + '/users/login'
PROFILE_URL = BASE_URL + VERSION + '/users/me'
DELIVERY_TIME_SLOTS_URL = BASE_URL + VERSION + '/stores/slots?storeId={storeId}&fulfilment=homeDelivery'
PICK_UP_TIME_SLOTS_URL = BASE_URL + VERSION + '/stores/slots?storeId={storeId}&fulfilment=collection'
ORDERS_URL = BASE_URL + VERSION + '/users/me/orders'
BASKET_URL = BASE_URL + VERSION + '/basket?withMOV=false'

DEFAULT_HEADERS = {}

REFRESH_RATE = 120

_LOGGER = logging.getLogger(__name__)

# ERROR CODES
# 4014 = Invalid Username and/or Password
# 4019 = No Slots For Date Requested


class JumboApi(object):
    """ Interface class for the Jumbo API """

    def __init__(self, user, password, refresh_rate=REFRESH_RATE):
        """ Constructor """

        self._user = user
        self._password = password
        self._profile = None
        self._open_deliveries = {}
        self._open_pick_ups = {}
        self._delivery_time_slots = []
        self._pick_up_time_slots = []
        self._basket = None
        self._jumbo_token = None
        self._last_refresh = None
        self._refresh_rate = refresh_rate
        self._request_login()

    def _update(self):
        """ Update the cache """
        current_time = int(time.time())
        last_refresh = 0 if self._last_refresh is None else self._last_refresh

        if current_time >= (last_refresh + self._refresh_rate):
            self._update_profile()
            self._update_orders()
            self._update_basket()
            self._update_time_slots()
            self._last_refresh = int(time.time())

    def _update_profile(self):
        """ Retrieve profile """
        response = self._request_update(PROFILE_URL)

        if 'user' not in response:
            return

        if 'data' not in response['user']:
            return

        self._profile = Profile(response['user']['data'])

    def _update_orders(self):
        """ Retrieve orders """
        response = self._request_update(ORDERS_URL)

        if 'orders' not in response:
            return

        if 'data' not in response['orders']:
            return

        self._open_deliveries = {}

        for order in response['orders']['data']:
            # TODO: I am not sure if "READY_TO_PICK_UP" is a status for pick ups
            if order['status'] in ["OPEN", "PROCESSING", "READY_TO_DELIVER", "READY_TO_PICK_UP"]:
                # DELIVERIES
                if order['type'] == "homeDelivery":
                    self._open_deliveries[order['id']] = Delivery(order)
                # PICK UPS
                elif order['type'] == "collection":
                    self._open_pick_ups[order['id']] = PickUp(order)

    def _update_basket(self):
        """ Retrieve basket """
        response = self._request_update(BASKET_URL)

        if 'basket' not in response:
            return

        if 'data' not in response['basket']:
            return

        self._basket = Basket(response['basket']['data'])

    def _update_time_slots(self):
        """ Update all time slots """
        self._update_delivery_time_slots()
        self._update_pick_up_time_slots()

    def _update_delivery_time_slots(self):
        """ Retrieve delivery time slots """
        response = self._request_update(DELIVERY_TIME_SLOTS_URL.format(storeId=self._profile.store.id))

        if 'timeSlots' not in response:
            return

        if 'data' not in response['timeSlots']:
            return

        self._delivery_time_slots = []

        for day in response['timeSlots']['data']:
            for time_slot in day['timeSlots']:
                self._delivery_time_slots.append(TimeSlot(time_slot))

    def _update_pick_up_time_slots(self):
        """ Retrieve pick up time slots """
        response = self._request_update(PICK_UP_TIME_SLOTS_URL.format(storeId=self._profile.store.id))

        if 'timeSlots' not in response:
            return

        if 'data' not in response['timeSlots']:
            return

        self._pick_up_time_slots = []

        for day in response['timeSlots']['data']:
            for time_slot in day['timeSlots']:
                self._pick_up_time_slots.append(TimeSlot(time_slot))

    def get_profile(self):
        """ Get your personal profile """
        self._update()
        return self._profile

    def get_open_deliveries(self):
        """ Get all open deliveries """
        self._update()
        return self._open_deliveries.values()

    def get_open_pick_ups(self):
        """ Get all open pick ups """
        self._update()
        return self._open_pick_ups.values()

    def get_basket(self):
        """ Get your current basket """
        self._update()
        return self._basket

    def get_open_delivery_time_slots(self):
        """" Get current open delivery time slots """
        self._update()

        return [
            ts
            for ts in self._delivery_time_slots
            if ts.is_available
        ]

    def get_open_pick_up_time_slots(self):
        """" Get current open pick up time slots """
        self._update()

        return [
            ts
            for ts in self._pick_up_time_slots
            if ts.is_available
        ]

    def _request_update(self, url):
        """ Perform a request to update information """
        if self._jumbo_token is None:
            self._request_login()

        _LOGGER.debug("Requesting update from " + url)

        headers = {
            "x-jumbo-token": self._jumbo_token,
            "Content-Type": "application/json",
        }
        response = requests.request("GET", url, headers={**headers, **DEFAULT_HEADERS})

        if response.status_code != 200:
            data = response.json()
            if 'code' in data:
                _LOGGER.error("Error code: " + data['code'])
                _LOGGER.error("Reason: " + data['message'])
            else:
                _LOGGER.error("Unable to perform request. Response: " + data)
            return {}

        return response.json()

    def _request_login(self):
        headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
        }

        payload = {
            'username': self._user,
            'password': self._password
        }

        try:
            response = requests.request(
                'POST', AUTHENTICATE_URL, data=payload, headers=headers)
            data = response.json()
            headers = response.headers

        except Exception:
            raise (UnauthorizedException())

        if 'code' in data:
            raise UnauthorizedException(data['message'])

        if 'x-jumbo-token' not in headers:
            raise UnauthorizedException("Unknown error occurred: No token found")

        self._jumbo_token = headers['x-jumbo-token']


class UnauthorizedException(Exception):
    pass



from datetime import datetime

from objects.price import Price


class PickUp(object):
    def __init__(self, data):
        delivery_date = datetime.fromtimestamp(int(data.get("pickup").get("date")) / 1000)

        self.id = data.get("id")
        self.status = data.get("status").lower()
        self.date = delivery_date.strftime("%Y-%m-%d")
        self.time = data.get("pickup").get("time")
        self.start_time = datetime.fromtimestamp(int(data.get("pickup").get("startDateTime")) / 1000)
        self.end_time = datetime.fromtimestamp(int(data.get("pickup").get("endDateTime")) / 1000)
        self.price = Price(data.get("prices").get("total"))

    def __str__(self):
        return f"{self.id} {self.status} {self.date} {self.time} {self.start_time} {self.end_time} {self.price}"



from datetime import datetime

from objects.price import Price


class TimeSlot(object):
    def __init__(self, data):
        self.type = data.get("type").lower()
        self.start_date_time = datetime.fromtimestamp(int(data.get("startDateTime")) / 1000)
        self.end_date_time = datetime.fromtimestamp(int(data.get("endDateTime")) / 1000)
        self.available = data.get("available")
        self.price = None

        if self.is_available:
            self.price = Price(data.get("price"))

    @property
    def is_available(self):
        return self.available

    def __str__(self):
        return f"{self.type} {self.start_date_time} {self.end_date_time} {self.available} {self.price}"



class Store(object):
    def __init__(self, data):
        self.id = data.get("id")
        self.complex_number = data.get("complexNumber")
        self.longitude = data.get("longitude")
        self.latitude = data.get("latitude")

    def __str__(self):
        return f"{self.id} {self.complex_number} {self.longitude} {self.latitude}"



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='python-jumbo-api',
    version='0.5.4',
    description='Unofficial python wrapper for the Jumbo.com API, a way to view your orders, time slots and basket information',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://www.github.com/peternijssen/python-jumbo-api/',
    author='Peter Nijssen',
    author_email='peter@peternijssen.nl',
    license='MIT',
    install_requires=['requests>=2.0'],
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ),
)
